
# q-expansion of newform 1728.3.q.j, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1728
weight = 3
poly_data = [144, 144, 276, -84, 133, -2, 15, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[-256788, 140724, -29928, -75285, -7826, -5547, 198, -473], 159300], [[78912, 156924, -132153, 82215, -6901, 10353, -1827, 677], 159300], [[-177138, -98226, -29928, -75285, -7826, -5547, 198, -473], 79650], [[-401472, 247356, -232782, 143085, -19444, 16707, -2688, 1013], 159300], [[194364, 129078, -107691, 68730, -7447, 9966, -2019, 644], 79650], [[209808, -1245834, 475248, -389565, 47066, -45123, 7182, -2407], 159300], [[2015568, 544536, 364008, 657585, 95186, 67467, -4878, 5753], 318600]]

hecke_ring_character_values = [[703, [1, 0, 0, 0, 0, 0, 0, 0]], [325, [1, 0, 0, 0, 0, 0, 0, 0]], [1217, [0, 0, 1, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [-1, 1, 1, 0, -1, 0, 0, 0], [1, -1, -2, 0, 0, 0, 1, 0], [2, 1, 4, -3, 0, 2, 1, -2], [-1, 1, -3, -2, 3, 6, 2, -2], [0, -3, -4, 1, 1, 1, 4, -2], [2, 5, 0, 1, 0, 0, 0, -4], [14, -2, -8, 0, 4, 0, 3, 3], [-10, -2, -9, 1, 0, 3, -2, 4], [0, 0, -12, 0, -6, -12, -1, 1], [-16, -2, 0, 0, 6, -6, 0, 2], [-42, 0, 21, 0, 0, 0, -2, -2], [14, -7, -14, 7, -12, -6, -7, 0], [-15, -3, -10, -2, 0, 6, -3, 6], [-10, -2, 8, -4, 4, 4, 12, -6], [-21, -9, 6, 0, 14, 0, 3, 3], [-2, -12, -5, 5, -18, -9, 2, 0], [5, -5, 0, 10, 0, 0, -5, 5], [-42, 12, 84, -12, 4, 4, 0, 0], [-4, 11, 0, -7, -9, 9, 0, -18], [13, -5, -12, 6, 24, 12, -7, 0], [19, -1, 22, -2, 0, -2, -1, 2], [-2, 14, -16, -24, 0, 0, 20, -10], [-63, 10, 67, -6, 0, 0, 2, 0], [-36, 12, -21, -27, 0, -15, 12, -24], [-6, 6, 42, -12, -6, -12, 7, -7], [30, 3, -64, -5, 18, 18, 4, -2], [48, 20, 0, -18, -6, 6, 0, -38], [-93, -15, 39, 0, -3, 0, 8, 8], [78, -2, 0, -4, 6, -6, 0, -2], [12, 24, 6, 0, -38, 0, -7, -7], [3, -6, 19, -10, 0, -42, -6, 12], [-29, 29, 14, -58, 0, 0, 23, -23], [-19, -17, 1, 0, -35, 0, -10, -10], [-9, -17, 0, 8, -24, -12, 1, 0], [11, -11, 13, 22, -3, -6, 24, -24], [-6, 0, 0, 6, -30, 30, 0, 6], [102, -6, -54, 0, -42, 0, 3, 3], [-52, 16, -21, -47, 0, 15, 16, -32], [-8, -4, 48, 20, -42, -42, -32, 16], [36, 12, 0, -14, -12, 12, 0, -26], [-49, 1, -70, 20, 0, -12, 1, -2], [0, 0, -7, 0, 18, 36, 10, -10], [0, -24, -40, 4, 24, 24, 40, -20], [-64, -10, 0, -4, 30, -30, 0, 6], [14, -14, 8, 28, -24, -48, -35, 35], [47, 27, -42, -22, -12, -6, 17, 0], [-34, 25, -68, 9, 0, 12, 25, -50], [-11, 11, 5, -22, -33, -66, -8, 8], [-122, 23, 228, -31, 77, 77, 16, -8], [162, 42, -60, 0, 32, 0, -29, -29], [-3, 36, 35, -4, -12, -6, -28, 0], [-86, -25, 128, 3, -16, -16, 44, -22], [-10, -32, -11, 0, -28, 0, -16, -16], [-133, -11, -128, 6, 0, 82, -11, 22], [38, 10, -48, 4, 44, 44, -28, 14], [30, 2, 0, 0, -12, 12, 0, -2], [112, -66, -169, 9, 66, 33, 48, 0], [-40, 22, -27, -35, 0, -1, 22, -44], [12, -12, 142, 24, 6, 12, 17, -17], [-49, -47, 1, 0, -51, 0, 20, 20], [298, -1, 0, -9, 30, -30, 0, -8], [32, -32, -32, 0, 54, 0, 55, 55], [169, -52, -177, 44, 0, 0, -36, 0], [-12, -18, -29, 35, 0, -105, -18, 36], [-205, 33, 228, -10, -96, -48, -13, 0], [10, -10, 185, 20, 6, 12, -28, 28], [157, 53, -52, 0, -66, 0, 7, 7], [-80, 86, 147, -19, -90, -45, -48, 0], [15, -6, -13, 34, 0, -130, -6, 12], [8, -26, -80, -6, 108, 108, 64, -32], [173, 39, -108, 26, -96, -48, -91, 0], [-45, 45, 185, -90, 27, 54, -6, 6], [-352, 35, 0, -25, -24, 24, 0, -60], [186, -42, -114, 0, -54, 0, 3, 3], [240, 12, 295, -67, 0, 83, 12, -24], [108, 22, 0, 16, 42, -42, 0, -6], [244, 38, -103, 0, 84, 0, -10, -10], [-34, 34, -155, -68, -78, -156, 16, -16], [458, -14, -236, 0, -108, 0, 5, 5], [12, 56, 19, -25, -114, -57, -6, 0], [-184, -2, 336, -14, -104, -104, 32, -16], [-56, -33, 0, 33, -21, 21, 0, 66], [203, -29, -216, 16, 12, 6, -3, 0], [-212, -43, -138, -31, 0, -14, -43, 86], [40, 35, -100, -45, 179, 179, 20, -10], [185, 110, -129, -54, 36, 18, -2, 0], [96, -30, 87, 39, 0, 35, -30, 60], [50, -50, 172, 100, 12, 24, -21, 21], [-2, -73, 40, 91, 126, 126, -36, 18], [159, 21, 88, 50, 0, 146, 21, -42], [82, 36, 0, -98, -60, 60, 0, -134], [381, -87, -234, 0, 168, 0, 59, 59], [-25, 25, 466, -50, -48, -96, -77, 77], [-106, 70, 132, -110, 150, 150, 80, -40], [205, 71, -67, 0, -105, 0, -70, -70], [66, -39, -36, 87, 1, 1, -96, 48], [168, -34, 0, 4, 0, 0, 0, 38], [258, -108, 0, -24, 42, -42, 0, 84], [94, -119, -160, 53, 132, 66, 13, 0], [-18, -84, -48, 42, 22, 22, 84, -42], [561, 33, -264, 0, -40, 0, -65, -65], [-205, -20, -281, 96, 0, 28, -20, 40], [-47, 47, -268, -94, 36, 72, 35, -35], [360, -25, 0, -15, -27, 27, 0, 10], [-222, -3, -330, 111, 0, -166, -3, 6], [194, -26, -248, 96, -124, -124, -140, 70], [142, -94, -118, 0, -54, 0, 19, 19], [-203, 22, 183, -42, 216, 108, 62, 0], [-58, 58, -2, -116, 60, 120, -33, 33], [-40, 62, 0, 36, -30, 30, 0, -26], [184, 2, -91, 0, 48, 0, -38, -38], [-304, -19, 236, -49, 168, 84, 117, 0], [-666, -56, 0, -2, 6, -6, 0, 54], [257, -68, 175, 150, 0, 4, -68, 136], [31, -31, 0, 62, 18, 36, -93, 93], [-120, 30, 256, -22, -260, -260, -16, 8], [331, -91, -211, 0, 57, 0, 32, 32], [-45, 15, 84, -144, 0, -24, 15, -30], [-35, 35, -723, -70, 69, 138, 76, -76], [780, -52, -747, 85, -138, -69, -118, 0], [-70, 4, -45, -29, 0, 161, 4, -8], [50, -71, 32, 137, 10, 10, -132, 66], [459, 65, -362, 32, 84, 42, -129, 0], [-38, 26, 216, 44, 78, 78, -140, 70], [512, 114, -409, -11, 78, 39, -92, 0], [-64, -182, 176, 206, 74, 74, -48, 24], [-705, -93, 576, -36, -96, -48, 165, 0], [-127, 127, 393, -254, 3, 6, 166, -166], [328, 93, 0, 81, -114, 114, 0, -12], [58, 62, 2, 0, 28, 0, 79, 79], [-24, 24, 182, -48, 6, 12, 11, -11], [254, -88, 0, -56, -132, 132, 0, 32], [283, -79, -181, 0, -91, 0, 2, 2], [-29, 29, 655, -58, 9, 18, 120, -120], [10, 158, -24, -160, -18, -18, 4, -2], [0, 0, -142, 0, -102, -204, 13, -13], [1075, 29, -523, 0, -61, 0, -6, -6], [-108, -9, 148, -25, -71, -71, 68, -34], [-306, 17, 0, -43, 60, -60, 0, -60], [-34, -86, -57, 109, 0, -201, -86, 172], [-118, 118, -224, -236, 36, 72, 231, -231], [84, 72, -296, -136, -94, -94, 128, -64], [364, 22, 0, 44, -12, 12, 0, 22], [69, 9, 142, -82, 0, -60, 9, -18], [118, -140, -209, 49, -246, -123, 42, 0], [52, 74, 209, -231, 0, 27, 74, -148], [-57, 57, -412, -114, -108, -216, 83, -83], [-20, 122, 48, -118, 26, 26, -8, 4], [-5, 5, -43, -10, -195, -390, -70, 70], [134, 130, -192, -92, -68, -68, -76, 38], [238, 69, 0, 85, -78, 78, 0, 16], [156, 60, -48, 0, -172, 0, 35, 35], [542, -211, -706, 47, 192, 96, 117, 0], [33, 117, 0, -84, 0, 286, 117, -234], [-394, 158, 0, 152, 102, -102, 0, -6], [76, -46, 209, -87, 0, -145, -46, 92], [-336, 114, 0, -96, 24, -24, 0, -210], [465, -141, -303, 0, -9, 0, 68, 68], [478, -115, 420, 173, 0, -246, -115, 230], [-168, -15, 348, 21, 39, 39, -12, 6], [50, -50, -208, 100, 138, 276, 55, -55], [266, 118, -460, -82, 132, 132, -72, 36], [-808, 82, 445, 0, 0, 0, -16, -16], [-21, -75, 216, -162, 0, -48, -75, 150], [490, 78, 0, -44, -168, 168, 0, -122], [-740, 78, 851, 33, -18, -9, -144, 0], [284, 42, 0, -116, -36, 36, 0, -158], [-1279, -29, 625, 0, 23, 0, 74, 74], [-120, -123, -106, 109, 0, 24, -123, 246], [-43, 43, 525, -86, 117, 234, -18, 18], [-442, -98, 172, 0, 62, 0, -37, -37], [-457, 174, 535, -96, 36, 18, 18, 0], [228, -228, 46, 456, -96, -192, -357, 357], [-85, -71, 7, 0, -91, 0, 80, 80], [220, -47, -226, 41, -444, -222, -35, 0], [-94, 76, -160, -250, 150, 150, 348, -174], [-526, 140, 0, -34, -174, 174, 0, -174], [178, -134, -165, 147, 246, 123, -160, 0], [589, -81, -718, -48, 72, 36, 177, 0], [204, -27, 378, -147, 0, -56, -27, 54], [21, -21, -427, 42, 51, 102, 80, -80], [72, -93, -284, 23, 3, 3, 140, -70], [-962, -286, 338, 0, 108, 0, 59, 59], [572, 70, 519, -17, 0, -75, 70, -140], [-162, 206, 0, -186, -114, 114, 0, -392], [-73, 313, 246, -140, -36, -18, -33, 0], [104, -104, -723, 208, 72, 144, 88, -88], [688, -82, -1296, 122, -150, -150, -80, 40], [-312, -23, 0, 39, 51, -51, 0, 62], [36, 93, 100, -157, 0, 126, 93, -186], [546, 23, 0, -37, 84, -84, 0, -60], [-758, 32, -717, -73, 0, 245, 32, -64], [182, 205, -144, -95, -186, -186, -220, 110], [748, -298, -523, 0, 68, 0, 48, 48], [58, -58, 1, 116, -294, -588, -106, 106], [-668, -282, 555, 169, 78, 39, -56, 0], [355, 152, 435, -232, 0, 152, 152, -304], [300, -192, -752, 116, 30, 30, 152, -76], [-1141, 61, 601, 0, 229, 0, -64, -64], [583, -33, -582, 34, 612, 306, -35, 0], [-41, 41, 245, -82, 45, 90, 204, -204], [-1193, -236, 1071, 114, 132, 66, 8, 0], [-314, 83, 480, -157, 132, 132, 148, -74], [-722, 98, 1352, -144, 78, 78, 92, -46], [-1002, -160, 0, 22, 30, -30, 0, 182], [863, -101, -482, 0, -78, 0, -49, -49], [-679, 28, -457, -250, 0, -280, 28, -56], [-31, 31, -206, -62, 240, 480, 123, -123], [380, 133, 0, -49, 165, -165, 0, -182], [-719, 59, 389, 0, -375, 0, 6, 6], [657, 155, -646, -144, -288, -144, 133, 0], [583, -49, 508, 124, 0, -272, -49, 98], [-336, -84, 126, 0, -160, 0, -133, -133], [299, 188, -217, -106, -108, -54, 24, 0], [-32, 32, 664, -64, 54, 108, 231, -231], [-179, 2, -149, -32, 0, -74, 2, -4], [468, 96, -856, -56, -306, -306, -80, 40], [585, -117, -351, 0, 507, 0, 142, 142], [7, -7, 469, 14, -177, -354, -262, 262], [-40, 40, -690, -80, 114, 228, -153, 153], [350, 136, -107, 0, 18, 0, -190, -190], [858, 188, 0, -90, 84, -84, 0, -278], [-1795, 121, 958, 0, 24, 0, 3, 3], [154, 106, -121, -73, 162, 81, 40, 0], [-90, -12, 141, -219, 0, 135, -12, 24], [944, 142, -1848, -122, -150, -150, -40, 20], [1073, -125, -1148, 50, -12, -6, 25, 0], [291, 75, 4, 212, 0, 332, 75, -150], [-115, 115, 569, -230, -129, -258, 404, -404], [-1034, -80, 0, 22, 90, -90, 0, 102], [-202, 202, -392, -404, 126, 252, 147, -147], [1698, 300, -699, 0, -146, 0, -24, -24], [15, -147, 32, 194, -168, -84, -241, 0], [159, 81, -166, 244, 0, 116, 81, -162], [-213, 213, 991, -426, 249, 498, 6, -6], [1130, -140, -2232, 154, -74, -74, -28, 14], [-279, 69, 174, 0, 250, 0, -25, -25], [672, -72, -1080, 204, 170, 170, -264, 132], [-156, 21, -374, 197, 0, 84, 21, -42], [958, 84, 0, 142, -6, 6, 0, 58], [-44, 44, 470, -88, 6, 12, -201, 201], [196, -32, 0, 2, -300, 300, 0, 34], [-64, 10, 37, 0, -498, 0, -110, -110], [697, -91, 706, 82, 0, -436, -91, 182], [-1306, -310, 0, 100, 12, -12, 0, 410], [-205, -389, -92, 0, -60, 0, 71, 71], [94, -94, 1040, 188, -144, -288, -239, 239], [56, 148, -128, -156, -482, -482, 16, -8], [-71, 71, 621, -142, -153, -306, -78, 78], [720, -27, -1236, 129, -141, -141, -204, 102], [-1486, -14, 736, 0, 158, 0, -1, -1], [-334, 294, 601, -27, -666, -333, -240, 0], [282, -36, 243, 75, 0, 399, -36, 72], [-128, -1, -92, -173, 120, 120, 348, -174], [-488, 36, 0, 6, 90, -90, 0, -30], [-269, 177, 440, -6, 360, 180, -165, 0], [-4, 10, 488, 230, -348, -348, -480, 240], [760, -141, 0, 101, 321, -321, 0, 242], [-201, -409, -192, 16, -36, -18, 377, 0], [320, -107, 602, -175, 0, 18, -107, 214], [87, -87, -515, 174, -15, -30, -72, 72], [-204, 42, 0, 252, -12, 12, 0, 210], [-713, -283, 215, 0, -191, 0, 110, 110], [-964, 93, 924, -133, 480, 240, 173, 0], [194, -74, -288, 124, 416, 416, -100, 50], [-515, -88, -725, 298, 0, -400, -88, 176], [55, -55, -912, 110, -150, -300, -5, 5], [-897, -111, 393, 0, -455, 0, -34, -34], [-93, 93, -707, -186, 81, 162, 210, -210], [-66, -8, 0, 10, -426, 426, 0, 18], [445, -272, -581, 136, -204, -102, 0, 0], [-184, 184, 34, -368, -84, -168, -25, 25], [252, -252, -1275, 504, -12, -24, 94, -94], [-710, 214, 0, 40, 24, -24, 0, -174], [-964, 184, 574, 0, -726, 0, -155, -155], [416, -300, -505, 211, 66, 33, -122, 0], [-512, -561, 0, -79, 63, -63, 0, 482], [268, -139, 288, 119, 0, -124, -139, 278], [362, -194, -278, 0, -260, 0, 127, 127], [94, -94, 570, 188, -450, -900, -197, 197], [39, -15, -416, 470, 0, -14, -15, 30], [188, 126, -285, -223, -390, -195, 320, 0], [-3, 3, -786, -6, -18, -36, 7, -7], [-510, -18, 244, -370, 224, 224, 776, -388], [-516, -322, 0, 92, -120, 120, 0, 414], [527, -311, -419, 0, 231, 0, -56, -56], [603, 655, -106, -158, -72, -36, -339, 0], [-186, 147, 412, -127, 307, 307, -40, 20], [-42, -78, -205, 241, 0, -127, -78, 156], [932, 49, -1780, -7, 42, 42, -84, 42], [1678, 248, -715, 0, -350, 0, -72, -72], [-1391, -295, 548, 0, 240, 0, 171, 171], [1186, 110, -1073, 3, 198, 99, -116, 0], [973, 239, -367, 0, -619, 0, -20, -20], [-201, -29, 140, -32, 0, 0, 93, 0], [702, 126, 227, 349, 0, 279, 126, -252], [580, -124, -1424, -8, -30, -30, 264, -132], [1115, -183, -1118, 180, -1044, -522, -177, 0], [-36, 36, -619, -72, 168, 336, 88, -88], [-466, -170, 1304, 356, -346, -346, -372, 186], [-690, 380, 0, -194, -390, 390, 0, -574]]
